package oclib

import (
	"encoding/json"
	"slices"

	"cloud.o-forge.io/core/oc-lib/models/resources"
	"cloud.o-forge.io/core/oc-lib/models/resources/workflow/graph"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type AbstractWorkflow struct {
	resources.ResourceSet
	Graph    *graph.Graph      `bson:"graph,omitempty" json:"graph,omitempty"`
	Schedule *WorkflowSchedule `bson:"schedule,omitempty" json:"schedule,omitempty"`
}

func (w *AbstractWorkflow) isDCLink(link graph.GraphLink) bool {
	if slices.Contains(w.Datacenters, link.Destination.ID) || slices.Contains(w.Datacenters, link.Source.ID) {
		return true
	}
	return false
}

type Workflow struct {
	utils.AbstractObject
	AbstractWorkflow
}

func (d *Workflow) GetName() string {
	return d.Name
}

func (d *Workflow) GetAccessor() utils.Accessor {
	data := &WorkflowMongoAccessor{}
	data.SetLogger(utils.WORKFLOW)
	return data
}

func (dma *Workflow) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *Workflow) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}
