package workspace

import (
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/resources/data"
	"cloud.o-forge.io/core/oc-lib/models/resources/datacenter"
	"cloud.o-forge.io/core/oc-lib/models/resources/processing"
	"cloud.o-forge.io/core/oc-lib/models/resources/storage"
	w "cloud.o-forge.io/core/oc-lib/models/resources/workflow"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type WorkspaceMongoAccessor struct {
	utils.AbstractAccessor
}

func (wfa *WorkspaceMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return wfa.GenericDeleteOne(id, wfa)
}

func (wfa *WorkspaceMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	d := set.(*Workspace)
	d.DataResources = nil
	d.DatacenterResources = nil
	d.StorageResources = nil
	d.ProcessingResources = nil
	d.WorkflowResources = nil
	return wfa.GenericUpdateOne(set, id, wfa, &Workspace{})
}

func (wfa *WorkspaceMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	d := data.(*Workspace)
	d.DataResources = nil
	d.DatacenterResources = nil
	d.StorageResources = nil
	d.ProcessingResources = nil
	d.WorkflowResources = nil
	return wfa.GenericStoreOne(d, wfa)
}

func (wfa *WorkspaceMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return wfa.GenericStoreOne(data, wfa)
}

func (wfa *WorkspaceMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	var workflow Workspace
	res_mongo, code, err := mongo.MONGOService.LoadOne(id, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}
	res_mongo.Decode(&workflow)
	obj := &Workspace{Active: false}
	mongo.MONGOService.UpdateMultiple(obj, map[string]interface{}{"active": true}, wfa.GetType())
	obj = &Workspace{Active: true}
	wfa.UpdateOne(obj, id)

	if workflow.Datas != nil && len(workflow.Datas) > 0 {
		dataAccessor := &data.DataMongoAccessor{}
		for _, id := range workflow.Datas {
			d, _, e := dataAccessor.LoadOne(id)
			if e == nil {
				workflow.DataResources = append(workflow.DataResources, d.(*data.DataResource))
			}
		}
	}
	if workflow.Datacenters != nil && len(workflow.Datacenters) > 0 {
		dataAccessor := &datacenter.DatacenterMongoAccessor{}
		for _, id := range workflow.Datacenters {
			d, _, e := dataAccessor.LoadOne(id)
			if e == nil {
				workflow.DatacenterResources = append(workflow.DatacenterResources, d.(*datacenter.DatacenterResource))
			}
		}
	}
	if workflow.Storages != nil && len(workflow.Storages) > 0 {
		dataAccessor := &storage.StorageMongoAccessor{}
		for _, id := range workflow.Storages {
			d, _, e := dataAccessor.LoadOne(id)
			if e == nil {
				workflow.StorageResources = append(workflow.StorageResources, d.(*storage.StorageResource))
			}
		}
	}
	if workflow.Processings != nil && len(workflow.Processings) > 0 {
		dataAccessor := &processing.ProcessingMongoAccessor{}
		for _, id := range workflow.Processings {
			d, _, e := dataAccessor.LoadOne(id)
			if e == nil {
				workflow.ProcessingResources = append(workflow.ProcessingResources, d.(*processing.ProcessingResource))
			}
		}
	}
	if workflow.Workflows != nil && len(workflow.Workflows) > 0 {
		dataAccessor := &w.WorkflowResourceMongoAccessor{}
		for _, id := range workflow.Workflows {
			d, _, e := dataAccessor.LoadOne(id)
			if e == nil {
				workflow.WorkflowResources = append(workflow.WorkflowResources, d.(*w.WorkflowResource))
			}
		}
	}
	return &workflow, 200, nil
}

func (wfa WorkspaceMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []Workspace
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r)
	}
	return objs, 200, nil
}

func (wfa *WorkspaceMongoAccessor) Search(word string) ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.Search(word, []string{"name"}, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not store to db. Error: " + err.Error())
		return nil, code, err
	}
	var results []Workspace
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r)
	}
	return objs, 200, nil
}
